//
// TRAKT workstations dump file to UDPA datagrams converter by Artiom N.(cl)2012 
//

//------------------------------------------------------------------------------
#include <stdlib.h>
#include <libgen.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <string.h>
#include <time.h>
//------------------------------------------------------------------------------
#include "ssender.h"
//------------------------------------------------------------------------------
struct t_configuration config;
//------------------------------------------------------------------------------
void usage(const char *p_name)
{
   printf("%s\n%s - %s\n%s\n\n", LINE_E, p_name, TITLE, LINE_E);
   printf("Usage:\n");
   printf("%s [-a <addr>] [-p <port>] [filename]\n\n", p_name);
   printf("Options:\n");
   printf("-a <address>\t - IP address for the sendeing (default: \"%s\")\n",
      ADDR);
   printf("-p <port>\t - send to port (default: \"%d\")\n",
      PORT);
   printf("filename\t - file with messages (if not specified: stdin)\n");
   printf("-h\t\t - print this help\n");
   //printf("%s\n", LINE_E, TITLE, LINE_E);
   return;
}
//------------------------------------------------------------------------------
void exitf()
{
   printf("%s\nExiting...\n", LINE_E);
   close(config.sock);
   if (config.fl) fclose(config.fl);
}
//------------------------------------------------------------------------------
void s_handler(int s)
{
   if (config.exec_status)
   {
      fprintf(stderr, "Retry Ctrl+C to exit immediatly...\n");
      config.exec_status = false;
   }
   else exit(EXIT_SUCCESS);
//   exit(EXIT_SUCCESS);
}
//------------------------------------------------------------------------------
int load_message(struct t_raw_msg *msg, FILE *fl)
// Read message from file descriptor.
// Return message length, if ok or code < 0, if fail.
{
   unsigned short int   msg_len  = 0;
   size_t               bc       = 0;
   size_t		mst	 = sizeof(msg_len);

   if (!fread(&msg_len, mst, 1, fl))
   {
      perror("Message length reading error!");
      return(-1);
   }

   //fseek(fl, -((long)sizeof(msg_len)), SEEK_CUR);
   free(msg->msg_data);
   msg->msg_data = NULL;
   msg->msg_size = msg_len;

   if (msg_len == 0) return(-2);

   if ((msg->msg_data = malloc(msg_len)) == NULL)
   {
      perror("Memory allocation for the message buffer error!");
      return(-3);
   }

   ((unsigned short int*)msg->msg_data)[0] = msg_len;

   if ((bc = fread(msg->msg_data + mst, 1, msg_len - mst, fl))
      != msg_len - mst)
   {
      perror("Message data reading error!");
      fprintf(stderr,
         "%d bytes was readed from stream, but message length %d bytes\n",
         bc + mst, msg_len
      );
         
      return(-4);
   }

   return(msg_len);
}
//------------------------------------------------------------------------------
int main(int argc, char **argv)
{
   int                  opt         = 0;
   unsigned int         port        = PORT;
   char                 *addr       = NULL;
   long                 msg_num     = 0;
   long                 bs          = 0;
   struct sockaddr_in   m_addr;

   if (atexit(&exitf) != 0)
   {
      perror("Can't register on exit function!");
      return(EXIT_FAILURE);
   }

   memset(&config, 0, sizeof(struct t_configuration));

   signal(SIGINT, &s_handler);
   #ifndef WINDOWS
   signal(SIGQUIT, &s_handler);
   signal(SIGKILL, &s_handler);
   #endif

   while ((opt = getopt(argc, argv, "a:p:h")) != -1)
   {
      switch (opt)
      {
         case 'a':
            addr = strdup(optarg);
         break;
         case 'p':
            port = atoi(optarg);
         break;
         case 'h':
            usage((char*)basename(argv[0]));
            return(EXIT_SUCCESS);
         break;
         default:
            usage((char*)basename(argv[0]));
            return(EXIT_FAILURE);
         break;
      }
   }

   if (!addr) addr = strdup(ADDR);

   printf("%s\n%s\n\n", TITLE, LINE_E);
   printf("Initializing...\n");

   printf("Opening file stream...\n");

   if (optind >= argc) config.fl = fdopen(dup(fileno(stdin)), "r");
   else if ((config.fl = fopen(argv[optind], "r")) == NULL)
   {
      perror("File opening error!");
      return(EXIT_FAILURE);
   }

   printf("Network initialization...\n");

   if ((config.sock = socket(PF_INET, SOCK_DGRAM, 0)) < 0)
   {
      perror("Socket error!");
      return(EXIT_FAILURE);
   }

   memset(&m_addr, 0, sizeof(m_addr));
   m_addr.sin_family = AF_INET;
   m_addr.sin_port   = htons(port);

   if (inet_aton(addr, &m_addr.sin_addr) == 0)
   {
      free(addr);
      perror("Address resolution error");
      return(EXIT_FAILURE);
   }


/*   if (bind(config.sock, (struct sockaddr*)&m_addr, sizeof(m_addr)) < 0)
   {
      free(addr);
      perror("Binding error!");
      return(EXIT_FAILURE);
   }   */

   config.exec_status = true;
   printf("Network initialized...\nAddress: %s:%d\n", addr, port);
   printf("Message sending started...\n\n");
   free(addr);

   while(!feof(config.fl) && config.exec_status)
   {
      if (load_message(&config.msg, config.fl) < 0)
      {
         fprintf(stderr, "Loading message from file error!\n");
         //return(EXIT_FAILURE);
      }
      if (!config.exec_status) break;
      if (config.msg.msg_size == 0) continue;

      printf("Sending message %ld (size: %d; type: 0x%04x)...\n",
         msg_num++,
         config.msg.msg_size,
         *((unsigned short*)config.msg.msg_data + 1));


      if ((bs = sendto(config.sock, config.msg.msg_data, config.msg.msg_size, 0,
         (struct sockaddr*)&m_addr, sizeof(m_addr))) < 0)
      {
         perror("Message sending error!");
      }
      else
      {
         if (bs != config.msg.msg_size)
         {
            fprintf(stderr,
               "Message sending error. Message length is %d bytes, but %ld bytes was sended.\n",
               config.msg.msg_size, bs);
         }
         else printf("Message %ld was sended successfully.\n", msg_num);
      }
   }

   return(EXIT_SUCCESS);
}
//------------------------------------------------------------------------------

